#!/bin/sh

echo 0 > /sys/class/graphics/fbcon/cursor_blink

CAM_BW_FMT=Y8
CAM_DRIVER="vm050"
CAM_ENTITY_NUMBER=`media-ctl -p | grep " $CAM_DRIVER" | awk '{print $4}'`
CAM_I2C_ADRESS=$(find /sys/bus/i2c/drivers/phytec-vm050/ -name "?-00*" | cut -d / -f 7 | cut -c5-6)
CAM_I2C_BUS=$(find /sys/bus/i2c/drivers/phytec-vm050/ -name "?-00*" | cut -d / -f 7 | cut -c1-1)
CAM_DEVICE=$(media-ctl -e "$CAM_DRIVER@$CAM_I2C_ADRESS")
IPU1_CSI0_DEVICE=$(media-ctl -e "ipu1_csi0 capture")
IPU2_CSI1_DEVICE=$(media-ctl -e "ipu2_csi1 capture")
CAMERA_TYP=$(i2cget -y -f $CAM_I2C_BUS 0x$CAM_I2C_ADRESS 0x00 w | cut -c 4)

if [ "$CAMERA_TYP" = "1" ]; then
	CAMERA="VM-050"
	GRAB_RES="32x32"
	FRAME_SIZE=",width=32,height=32"
	FRAME_SCALE_SIZE=",width=320,height=320"
	OFFSET="(0,0)"
elif [ "$CAMERA_TYP" = "2" ]; then
	CAMERA="VM-051"
	GRAB_RES="80x64"
	FRAME_SIZE=",width=80,height=64"
	FRAME_SCALE_SIZE=",width=480,height=384"
	OFFSET="(0,0)"
else
	echo " no camera driver loaded"
	echo " please check camera device tree" 
	echo ""
	return 1
fi

echo ""
echo " found camera = $CAMERA"
echo " camera entity name = $CAM_ENTITY_NUMBER"
echo " camera subdevice name = $CAM_DEVICE"
echo " ipu1_csi0 capture device name = $IPU1_CSI0_DEVICE"
echo " ipu2_csi1 capture device name = $IPU2_CSI1_DEVICE"
echo ""

echo ""
echo " configure IPU1_CSI0 (camera_0 port) with media_control"
echo " ======================================================"

media-ctl -r
media-ctl -l "\"$CAM_ENTITY_NUMBER\":0 -> \"ipu1_csi0_mux\":1 [1]"
media-ctl -l "\"ipu1_csi0_mux\":2 -> \"ipu1_csi0\":0 [1]"
media-ctl -l "\"ipu1_csi0\":2 -> \"ipu1_csi0 capture\":0 [1]"

media-ctl -V "\"$CAM_ENTITY_NUMBER\":0[fmt:$CAM_BW_FMT/$GRAB_RES $OFFSET/$GRAB_RES]"
media-ctl -V "\"ipu1_csi0_mux\":2[fmt:$CAM_BW_FMT/$GRAB_RES field: none]"
media-ctl -V "\"ipu1_csi0\":2[fmt:$CAM_BW_FMT/$GRAB_RES field: none]"

echo ""
echo " configure camera with v4l2_control"
echo " =================================="

# Setze V4L2-Controlls
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=autoscale_mode=1	# Reset
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=autoscale_mode=5	# Set to: scale by TRACK_TEMP0
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=tracking_temperature=0	# Reset
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=tracking_temperature=2969	# Set TRACK_TEMP0 to 23,7°C
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=average=0	# Reset
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=average=2	# Enable medium average
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=filter=0	# Reset
#v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=filter=8	# Enable 3x3 median filter
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=horizontal_flip=1	# Reset
v4l2-ctl -d $IPU1_CSI0_DEVICE --set-ctrl=horizontal_flip=0	# Flip image (mirror)

echo ""
echo " start gstreamer, break with ctl-C"
echo " ================================= "

# Starte Bildausgabe mit gstreamer im Hintergrund
gst-launch-1.0 \
	v4l2src device=$IPU1_CSI0_DEVICE ! \
	video/x-raw,format=GRAY8,depth=8$FRAME_SIZE ! \
	pseudocolor2 location=/usr/share/gstpseudocolor/ironblack.rgb mode=table invert=0 ! video/x-raw,format=RGB$FRAME_SIZE ! \
	videoscale ! video/x-raw$FRAME_SCALE_SIZE ! \
	videoconvert ! \
	fbdevsink sync=true &

# Terminal für Textausgabe konfigurieren
clear >/dev/tty0
echo -e "\033[1m" >/dev/tty0		# Textfarbe setzen
echo -e "\033[33m" >/dev/tty0		# Hintergrundfarbe setzen
echo -e "\033[44m" >/dev/tty0		# Hintergrundfarbe setzen

# Skriptvariablen initiieren
i=0
j=0
k=0
l=0
fifo_full=0
temp_max_index=0
face_locked=0
temp_max_avg=0
temp_max_avg_old=0
messung_beendet=0
counter=0
coffeecounter=0
illcounter=0
increasecounter=0
avgcounter=0

blue_led=/sys/class/leds/blue\:user3/brightness
green_led=/sys/class/leds/green\:user2/brightness
red_led=/sys/class/leds/red\:user1/brightness

# RGB-LED initiieren
echo 0 > $blue_led
echo 0> $green_led
echo 255 > $red_led

# Tracking Pixel konfigurieren
i2cset -y -f 0 0x4a 0x21 0x8b21 w	# Byteswap: 0x218b = Pixelnr.: 395, Trackingmode: 1
i2cset -y -f 0 0x4a 0x22 0x9321 w	# Byteswap: 0x2193 = Pixelnr.: 403, Trackingmode: 1
i2cset -y -f 0 0x4a 0x23 0x8f22 w	# Byteswap: 0x228f = Pixelnr.: 655, Trackingmode: 1


# Begin Hauptschleife
while [ true ]
do
#	Temperaturen der Trackingpixel auslesen und Temperatur umerechnen in dezi-°C
	temp_acc[0]=$(i2cdump -y -f -r 0x25-0x27 0 0x4a w | tail -1 |awk '{print$2,$3,$4}')
	temp=${temp_acc[0]}
	temp_acc[0]=${temp:2:2}${temp:0:2}
	temp_acc[0]=$(( 16#${temp_acc[0]} - 2732))
	temp_acc[1]=${temp:7:2}${temp:5:2}
	temp_acc[1]=$(( 16#${temp_acc[1]} - 2732))
	temp_acc[2]=${temp:12:2}${temp:10:2}
	temp_acc[2]=$(( 16#${temp_acc[2]} - 2732))
	
	if [ $face_locked -eq 1 ]
	then	# Max Temp in dezi-°C
		temp_acc[4]=$(i2cget -y -f 0 0x4a 0x1c w | tail -1 | sed 's/0x//g')
		temp=${temp_acc[4]}
		temp_acc[4]=${temp:3:2}${temp:0:2}
		temp_acc[4]=$(( 16#${temp_acc[4]} - 2732))
	else	# TA in dezi-°C
		temp_acc[3]=$(i2cget -y -f 0 0x4a 0x72 w | tail -1 | sed 's/0x//g')
		temp=${temp_acc[3]}
		temp_acc[3]=${temp:3:2}${temp:0:2}
		temp_acc[3]=$(( 16#${temp_acc[3]} - 2732))
	fi

#	Ringspeicher fuellen, i wird bei jedem durchlauf der Hauptschleife um +1 erhöht
	temp1[$i]=${temp_acc[0]}
	temp2[$i]=${temp_acc[1]}
	temp3[$i]=${temp_acc[2]}
	temp_ta[$i]=${temp_acc[3]}
	temp_max[$i]=${temp_acc[4]}
		

#	Trackingpixel mitteln
	if [ $fifo_full -eq 1 ]	# Nach mindestens 10 durchläufen der Hauptschleife wird $fifo_full auf 1 gesetzt. Erst jetzt kann ein Gesicht erkannt werden.
		then
		temp1_avg=0
		temp2_avg=0
		temp3_avg=0
		temp_ta_avg=0

		for ((j=0; j<9; j++))
		do
			temp1_avg=$(( $temp1_avg + ${temp1[$j]} ))
			temp2_avg=$(( $temp2_avg + ${temp2[$j]} ))
			temp3_avg=$(( $temp3_avg + ${temp3[$j]} ))
			temp_ta_avg=$(( $temp_ta_avg + ${temp_ta[$j]} ))
		done
		temp1_avg=$(( $temp1_avg / 10 ))
		temp2_avg=$(( $temp2_avg / 10 ))
		temp3_avg=$(( $temp3_avg / 10 ))
		temp_ta_avg=$(( $temp_ta_avg / 10 ))

#		User-LED umschalten wenn Gesicht erkannt
		# Ein Tracking Pixel oder der Mittelwert eines Tracking Pixel is kleiner als der Mittelwert der Sensortemperatur ($temp_ta_avg)
		if [ $temp1_avg -lt $temp_ta_avg -o $temp2_avg -lt $temp_ta_avg -o $temp3_avg -lt $temp_ta_avg -o ${temp1[$i]} -lt $temp_ta_avg -o ${temp2[$i]} -lt $temp_ta_avg  -o ${temp3[$i]} -lt $temp_ta_avg ]
			then
			echo 255 > $red_led
			echo 0 > $green_led
			echo 0 > $blue_led
			face_locked=0
			if [ $messung_beendet -ne 0 ]
				then
				((messung_beendet--))
			fi
		
		# Alle Mittelwerte der Tracking Pixel sind größer als der Mittelwert der Sensortemperatur ($temp_ta_avg)
		elif [ $temp1_avg -gt $temp_ta_avg -a $temp2_avg -gt $temp_ta_avg -a $temp3_avg -gt $temp_ta_avg -a $messung_beendet -eq 0 ]
			then
			echo 0 > $red_led
			echo 255 > $green_led
			echo 0 > $blue_led
			((face_locked++))
		fi

	fi
	
	if [ $face_locked -gt 9 -a $k -eq 0 ] # Wenn Gesicht 10 hintereinander erkannt wurde beginnt die Messung. Die gemessene Höchsttemperatur wird akkumuliert und der Durchschnitt gebildet
		then
		clear >/dev/tty0
		echo "Gesicht erkannt, Messung läuft, nicht bewegen!" >/dev/tty0
#		Average zurucksetzen
		temp_max_acc=${temp_max[$i]}
	elif [ $face_locked -gt 10 -a $k -ne 0 ]
		then
		temp_max_acc=$(( $temp_max_acc + ${temp_max[$i]} ))
		temp_max_avg=$(( $temp_max_acc / ($k + 1) ))
	fi
	
	((k++))
	
	if [ $face_locked -le 9 ]
		then
		k=0
	fi
	
	if [ $temp_max_avg -ne $temp_max_avg_old -o $k -lt 29 ] # Wenn 30 Messwerte vorhanden sind und der Mittelwert stabil ist, wird die Temperatur ausgewertet
		then
		temp_max_avg_old=$temp_max_avg
	elif [ $face_locked -gt 1 -a $messung_beendet -eq 0 ]
		then
		messung_beendet=5	# timeout=5s
		
#		Die letzten Durchschnittswerte mitteln
		temp_max_fifo_avg=0
		for (( j=0; j<temp_max_index ;j++ ))
		do
			temp_max_fifo_avg=$(( $temp_max_fifo_avg + ${temp_max_fifo[$j]} ))
		done
		if [ $temp_max_index -eq 0 ]	# Sonderfall 1. Messung
			then
			temp_max_fifo_avg=$temp_max_avg
			((j++))
		else
			temp_max_fifo_avg=$(( $temp_max_fifo_avg / j ))
		fi
		
		echo "Messung beendet." >/dev/tty0
		clear >/dev/tty0
		echo "Deine Haut-Temperatur: ${temp_max_avg:0:2},${temp_max_avg:2:1}°C" >/dev/tty0
		echo "Der Durchschnitt der letzen $j Messungen ist: ${temp_max_fifo_avg:0:2},${temp_max_fifo_avg:2:1}°C" >/dev/tty0
		
		if [ $(( $temp_max_avg - $temp_max_fifo_avg )) -ge 40 ] # Wenn die abweichung vom Durchschnitt der letzten Messungen größer größer als 4,0°C ist, wird der Messwert verworfen.
			then
			echo "Dein Kaffee ist warm genug :-P" >/dev/tty0
			((coffeecounter++))
			echo $coffeecounter > /home/root/coffeecounter.log

		else
#			Aktuellen Messwert im Ringspeicher ablegen
			temp_max_fifo[$l]=$temp_max_avg
			
			if [ $l -lt 9 ]
				then
				((l++))
			else
				l=0
				temp_max_index=10
			fi
			if [ $temp_max_index -lt 10 ]
				then
				temp_max_index=$l
			fi
			
			if [ $(( $temp_max_avg - $temp_max_fifo_avg )) -ge 20 ]
				then
				echo "Deine Temperatur ist deutlich höher als der Durchschnitt. Du solltest später Fieber messen." >/dev/tty0
				((illcounter++))
				echo $illcounter > /home/root/illcounter.log
				
			elif [ $(( $temp_max_avg - $temp_max_fifo_avg )) -ge 15 ]
				then
				echo "Deine Temperatur ist etwas höher als der Durchschnitt. Du solltest deine Temperatur später erneut messen." >/dev/tty0
				((increasecounter++))
				echo $increasecounter > /home/root/increasecounter.log
			else
				echo "Super, Deine Temperatur liegt im Bereich des Durchschnitt +/- 1,5°C." >/dev/tty0
				((avgcounter++))
				echo $avgcounter > /home/root/avgcounter.log
			fi
		fi

		echo "Bitte gehe aus dem Bild. Die nächste Messung kann in ca. 5 Sekunden gestartet werden." >/dev/tty0
		((counter++))
		
		# Zählervariablen ausgeben
		echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0; echo >/dev/tty0;
		echo "$counter : $avgcounter : $increasecounter : $illcounter : $coffeecounter" >/dev/tty0
		echo $counter > /home/root/counter.log
	fi


#	Ringspeicherindex zuruecksetzen wenn am letzten Ringspeicherindex
	if [ $i -lt 9 ]
		then
			((i++))
		else
			i=0
			fifo_full=1
	fi

	sleep 0.02

done	# Ende Hauptschleife



echo -e "\033[0m"		# Alle Texteigenschaften zurücksetzen

echo 1 > /sys/class/graphics/fbcon/cursor_blink

